"""AIMinO Frontend Package.

This package contains the Napari frontend application for AIMinO, including:
- aimino_core: Command models, handlers, and executor
- napari_app: Napari application entry points and agent client
"""

__version__ = "0.1.0"

__all__ = ["__version__"]

